@@ -43,6 +43,9 @@ $SBCL_XC_HOST <<-'EOF' || exit 1
 	(when (find :sb-test *shebang-features*)
 	  (load "tests/type.before-xc.lisp")
 	  (load "tests/info.before-xc.lisp"))
+        (load "tools-for-build/ucd.lisp")
+        (sb-cold::slurp-ucd)
+        (sb-cold::output)
         (host-cload-stem "src/compiler/generic/genesis")
 	(sb!vm:genesis :c-header-dir-name "src/runtime/genesis")
         #+cmu (ext:quit)
