@@ -15,10 +15,6 @@
 
 echo //entering make-target-1.sh
 
-# the GNU dialect of "make" -- easier to find or port it than to
-# try to figure out how to port to the local dialect..
-gnumake=${GNUMAKE:-gmake}
-
 # Build the runtime system and symbol table (.nm) file.
 #
 # (This C build has to come after the first genesis in order to get
@@ -27,15 +23,21 @@ gnumake=${GNUMAKE:-gmake}
 # could come either before or after running the cross compiler; that
 # doesn't matter.)
 echo //building runtime system and symbol table file
-cd src/runtime
-$gnumake clean  || exit 1
-$gnumake depend || exit 1
-$gnumake all    || exit 1
+cd ./src/runtime
+$GNUMAKE clean  || exit 1
+$GNUMAKE depend || exit 1
+$GNUMAKE all    || exit 1
 cd ../..
 
 # Use a little C program to grab stuff from the C header files and
 # smash it into Lisp source code.
-cd tools-for-build
-$gnumake grovel_headers
+cd ./tools-for-build
+$GNUMAKE -I../src/runtime grovel-headers || exit 1
+cd ..
+tools-for-build/grovel-headers > output/stuff-groveled-from-headers.lisp
+
+# after-grovel-headers may not exist for all platforms (used for
+# Darwin hacks)
+cd ./src/runtime
+$GNUMAKE after-grovel-headers || true
 cd ..
-tools-for-build/grovel_headers > output/stuff-groveled-from-headers.lisp
