@@ -352,9 +352,8 @@
 (defmacro-mundanely with-condition-restarts
     (condition-form restarts-form &body body)
   #!+sb-doc
-  "WITH-CONDITION-RESTARTS Condition-Form Restarts-Form Form*
-   Evaluates the Forms in a dynamic environment where the restarts in the list
-   Restarts-Form are associated with the condition returned by Condition-Form.
+  "Evaluates the BODY in a dynamic environment where the restarts in the list
+   RESTARTS-FORM are associated with the condition returned by CONDITION-FORM.
    This allows FIND-RESTART, etc., to recognize restarts that are not related
    to the error currently being debugged. See also RESTART-CASE."
   (let ((n-cond (gensym)))
@@ -379,7 +378,7 @@
 						 binding
 						 :test #'eq))
 			       (warn "Unnamed restart does not have a ~
-					report function: ~S"
+                                      report function: ~S"
 				     binding))
 			     `(make-restart :name ',(car binding)
 					    :function ,(cadr binding)
