@@ -17,23 +17,24 @@
 (defun style-warn (format-control &rest format-arguments)
   (/show0 "entering STYLE-WARN")
   (/show format-control format-arguments)
-  (warn 'simple-style-warning
-	:format-control format-control
-	:format-arguments format-arguments))
+  (with-sane-io-syntax
+      (warn 'simple-style-warning
+            :format-control format-control
+            :format-arguments format-arguments)))
 
 ;;; a utility for SIGNAL, ERROR, CERROR, WARN, COMPILER-NOTIFY and
 ;;; INVOKE-DEBUGGER: Parse the hairy argument conventions into a
 ;;; single argument that's directly usable by all the other routines.
 (defun coerce-to-condition (datum arguments default-type fun-name)
   (cond ((typep datum 'condition)
-	 (if arguments
-	     (cerror "Ignore the additional arguments."
-		     'simple-type-error
-		     :datum arguments
-		     :expected-type 'null
-		     :format-control "You may not supply additional arguments ~
-				     when giving ~S to ~S."
-		     :format-arguments (list datum fun-name)))
+	 (when (and arguments (not (eq fun-name 'cerror)))
+           (cerror "Ignore the additional arguments."
+                   'simple-type-error
+                   :datum arguments
+                   :expected-type 'null
+                   :format-control "You may not supply additional arguments ~
+                                    when giving ~S to ~S."
+                   :format-arguments (list datum fun-name)))
 	 datum)
 	((symbolp datum) ; roughly, (SUBTYPEP DATUM 'CONDITION)
 	 (apply #'make-condition datum arguments))
@@ -75,8 +76,8 @@
    (source :initarg :source :reader program-error-source))
   (:report (lambda (condition stream)
 	     (format stream "Execution of a form compiled with errors.~%~
-                            Form:~%  ~A~%~
-                            Compile-time-error:~%  ~A"
+                             Form:~%  ~A~%~
+                             Compile-time-error:~%  ~A"
 		       (program-error-source condition)
 		       (program-error-message condition)))))
 
