@@ -73,7 +73,7 @@
   #!+sb-doc
   "the standard structure for the description of a package"
   ;; the name of the package, or NIL for a deleted package
-  (%name nil :type (or simple-base-string null))
+  (%name nil :type (or simple-string null))
   ;; nickname strings
   (%nicknames () :type list)
   ;; packages used by this package
@@ -99,7 +99,7 @@
   ;; shadowing symbols
   (%shadowing-symbols () :type list)
   ;; documentation string for this package
-  (doc-string nil :type (or simple-base-string null))
+  (doc-string nil :type (or simple-string null))
   ;; package locking
   #!+sb-package-locks
   (lock nil :type boolean)
@@ -290,7 +290,7 @@
 	      (error 'simple-program-error
 		     :format-control
 		     "At least one of :INTERNAL, :EXTERNAL, or ~
-		      :INHERITED must be supplied."))
+                      :INHERITED must be supplied."))
 	   ,(dolist (symbol symbol-types)
 	      (unless (member symbol '(:internal :external :inherited))
 		(error 'program-error
