@@ -63,7 +63,7 @@
 ;;; walk.
 (declaim (ftype (function (node) (values)) check-node-reached))
 (defun check-node-reached (node)
-  (unless (gethash (continuation-block (node-prev node)) *seen-blocks*)
+  (unless (gethash (ctran-block (node-prev node)) *seen-blocks*)
     (barf "~S was not reached." node))
   (values))
 
@@ -332,82 +332,60 @@
 
   (let* ((fun (block-home-lambda block))
 	 (fun-deleted (eq (functional-kind fun) :deleted))
-	 (this-cont (block-start block))
+	 (this-ctran (block-start block))
 	 (last (block-last block)))
     (unless fun-deleted
       (check-fun-reached fun block))
-    (when (not this-cont)
+    (when (not this-ctran)
       (barf "~S has no START." block))
     (when (not last)
       (barf "~S has no LAST." block))
-    (unless (eq (continuation-kind this-cont) :block-start)
+    (unless (eq (ctran-kind this-ctran) :block-start)
       (barf "The START of ~S has the wrong kind." block))
 
-    (let ((use (continuation-use this-cont))
-	  (uses (block-start-uses block)))
-      (when (and (null use) (= (length uses) 1))
-	(barf "~S has a unique use, but no USE." this-cont))
-      (dolist (node uses)
-	(unless (eq (node-cont node) this-cont)
-	  (barf "The USE ~S for START in ~S has wrong CONT." node block))
-	(check-node-reached node)))
+    (when (ctran-use this-ctran)
+      (barf "The ctran ~S is used." this-ctran))
 
-    (let* ((last-cont (node-cont last))
-	   (cont-block (continuation-block last-cont))
-	   (dest (continuation-dest last-cont)))
-      (ecase (continuation-kind last-cont)
-	(:deleted)
-	(:deleted-block-start
-	 (let ((dest (continuation-dest last-cont)))
-	   (when dest
-	     (check-node-reached dest)))
-	 (unless (member last (block-start-uses cont-block))
-	   (barf "LAST in ~S is missing from uses of its Cont." block)))
-	(:block-start
-	 (check-node-reached (continuation-next last-cont))
-	 (unless (member last (block-start-uses cont-block))
-	   (barf "LAST in ~S is missing from uses of its Cont." block)))
-	(:inside-block
-	 (unless (eq cont-block block)
-	   (barf "CONT of LAST in ~S is in a different BLOCK." block))
-	 (unless (eq (continuation-use last-cont) last)
-	   (barf "USE is not LAST in CONT of LAST in ~S." block))
-	 (when (continuation-next last-cont)
-	   (barf "CONT of LAST has a NEXT in ~S." block))))
-
-      (when dest
-	(check-node-reached dest)))
+    (when (node-next last)
+      (barf "Last node ~S of ~S has next ctran." last block))
 
     (loop
-      (unless (eq (continuation-block this-cont) block)
-	(barf "BLOCK in ~S should be ~S." this-cont block))
+      (unless (eq (ctran-block this-ctran) block)
+	(barf "BLOCK of ~S should be ~S." this-ctran block))
 
-      (let ((dest (continuation-dest this-cont)))
-	(when dest
-	  (check-node-reached dest)))
-
-      (let ((node (continuation-next this-cont)))
+      (let ((node (ctran-next this-ctran)))
 	(unless (node-p node)
-	  (barf "~S has strange NEXT." this-cont))
-	(unless (eq (node-prev node) this-cont)
-	  (barf "PREV in ~S should be ~S." node this-cont))
+	  (barf "~S has strange NEXT." this-ctran))
+	(unless (eq (node-prev node) this-ctran)
+	  (barf "PREV in ~S should be ~S." node this-ctran))
 
+        (when (valued-node-p node)
+          (binding* ((lvar (node-lvar node) :exit-if-null))
+            (unless (memq node (find-uses lvar))
+              (barf "~S is not used by its LVAR ~S." node lvar))
+            (when (singleton-p (lvar-uses lvar))
+              (barf "~S has exactly 1 use, but LVAR-USES is a list."
+                    lvar))
+            (unless (lvar-dest lvar)
+              (barf "~S does not have dest." lvar))))
+
+        (check-node-reached node)
 	(unless fun-deleted
 	  (check-node-consistency node))
 
-	(let ((cont (node-cont node)))
-	  (when (not cont)
-	    (barf "~S has no CONT." node))
+	(let ((next (node-next node)))
+	  (when (and (not next) (not (eq node last)))
+	    (barf "~S has no NEXT." node))
 	  (when (eq node last) (return))
-	  (unless (eq (continuation-kind cont) :inside-block)
-	    (barf "The interior continuation ~S in ~S has the wrong kind."
-		  cont
+	  (unless (eq (ctran-kind next) :inside-block)
+	    (barf "The interior ctran ~S in ~S has the wrong kind."
+		  next
 		  block))
-	  (unless (continuation-next cont)
-	    (barf "~S has no NEXT." cont))
-	  (unless (eq (continuation-use cont) node)
-	    (barf "USE in ~S should be ~S." cont node))
-	  (setq this-cont cont))))
+	  (unless (ctran-next next)
+	    (barf "~S has no NEXT." next))
+	  (unless (eq (ctran-use next) node)
+	    (barf "USE in ~S should be ~S." next node))
+	  (setq this-ctran next))))
 
     (check-block-successors block))
   (values))
@@ -458,24 +436,18 @@
 
 ;;;; node consistency checking
 
-;;; Check that the DEST for CONT is the specified NODE. We also mark
-;;; the block CONT is in as SEEN.
-(declaim (ftype (function (continuation node) (values)) check-dest))
-(defun check-dest (cont node)
-  (let ((kind (continuation-kind cont)))
-    (ecase kind
-      (:deleted
-       (unless (block-delete-p (node-block node))
-	 (barf "DEST ~S of deleted continuation ~S is not DELETE-P."
-	       cont node)))
-      (:deleted-block-start
-       (unless (eq (continuation-dest cont) node)
-	 (barf "DEST for ~S should be ~S." cont node)))
-      ((:inside-block :block-start)
-       (unless (gethash (continuation-block cont) *seen-blocks*)
-	 (barf "~S receives ~S, which is in an unknown block." node cont))
-       (unless (eq (continuation-dest cont) node)
-	 (barf "DEST for ~S should be ~S." cont node)))))
+;;; Check that the DEST for LVAR is the specified NODE. We also mark
+;;; the block LVAR is in as SEEN.
+#+nil(declaim (ftype (function (lvar node) (values)) check-dest))
+(defun check-dest (lvar node)
+  (do-uses (use lvar)
+    (unless (gethash (node-block use) *seen-blocks*)
+      (barf "Node ~S using ~S is in an unknown block." use lvar)))
+  (unless (eq (lvar-dest lvar) node)
+    (barf "DEST for ~S should be ~S." lvar node))
+  (unless (find-uses lvar)
+    (barf "Lvar ~S has a destinatin, but no uses."
+          lvar))
   (values))
 
 ;;; This function deals with checking for consistency of the
@@ -494,28 +466,41 @@
 	     (check-fun-reached leaf node)))))
     (basic-combination
      (check-dest (basic-combination-fun node) node)
+     (when (and (mv-combination-p node)
+                (eq (basic-combination-kind node) :local))
+       (let ((fun-lvar (basic-combination-fun node)))
+         (unless (ref-p (lvar-uses fun-lvar))
+           (barf "function in a local mv-combination is not a LEAF: ~S" node))
+         (let ((fun (ref-leaf (lvar-use fun-lvar))))
+           (unless (lambda-p fun)
+             (barf "function ~S in a local mv-combination ~S is not local"
+                   fun node))
+           (unless (eq (functional-kind fun) :mv-let)
+             (barf "function ~S in a local mv-combination ~S is not of kind :MV-LET"
+                   fun node)))))
      (dolist (arg (basic-combination-args node))
        (cond
-	(arg (check-dest arg node))
-	((not (and (eq (basic-combination-kind node) :local)
-		   (combination-p node)))
-	 (barf "flushed arg not in local call: ~S" node))
-	(t
-	 (locally
-	   ;; KLUDGE: In sbcl-0.6.11.37, the compiler doesn't like
-	   ;; (DECLARE (TYPE INDEX POS)) after the inline expansion of
-	   ;; POSITION. It compiles it correctly, but it issues a type
-	   ;; mismatch warning because it can't eliminate the
-	   ;; possibility that control will flow through the
-	   ;; NIL-returning branch. So we punt here. -- WHN 2001-04-15
-	   (declare (notinline position))
-	   (let ((fun (ref-leaf (continuation-use
-				 (basic-combination-fun node))))
-		 (pos (position arg (basic-combination-args node))))
-	     (declare (type index pos))
-	     (when (leaf-refs (elt (lambda-vars fun) pos))
-	       (barf "flushed arg for referenced var in ~S" node)))))))
-     (let ((dest (continuation-dest (node-cont node))))
+         (arg (check-dest arg node))
+         ((not (and (eq (basic-combination-kind node) :local)
+                    (combination-p node)))
+          (barf "flushed arg not in local call: ~S" node))
+         (t
+          (locally
+              ;; KLUDGE: In sbcl-0.6.11.37, the compiler doesn't like
+              ;; (DECLARE (TYPE INDEX POS)) after the inline expansion of
+              ;; POSITION. It compiles it correctly, but it issues a type
+              ;; mismatch warning because it can't eliminate the
+              ;; possibility that control will flow through the
+              ;; NIL-returning branch. So we punt here. -- WHN 2001-04-15
+              (declare (notinline position))
+            (let ((fun (ref-leaf (lvar-use
+                                  (basic-combination-fun node))))
+                  (pos (position arg (basic-combination-args node))))
+              (declare (type index pos))
+              (when (leaf-refs (elt (lambda-vars fun) pos))
+                (barf "flushed arg for referenced var in ~S" node)))))))
+     (let* ((lvar (node-lvar node))
+            (dest (and lvar (lvar-dest lvar))))
        (when (and (return-p dest)
 		  (eq (basic-combination-kind node) :local)
 		  (not (eq (lambda-tail-set (combination-lambda node))
@@ -528,6 +513,8 @@
        (barf "IF not at block end: ~S" node)))
     (cset
      (check-dest (set-value node) node))
+    (cast
+     (check-dest (cast-value node) node))
     (bind
      (check-fun-reached (bind-lambda node) node))
     (creturn
@@ -594,7 +581,7 @@
 	  (unless (find-in #'tn-ref-next-ref target vop-refs)
 	    (barf "The target for ~S isn't in REFS for ~S." ref vop)))))))
 
-;;; Verify the sanity of the VOP-Refs slot in VOP. This involves checking
+;;; Verify the sanity of the VOP-REFS slot in VOP. This involves checking
 ;;; that each referenced TN appears as an argument, result or temp, and also
 ;;; basic checks for the plausibility of the specified ordering of the refs.
 (defun check-vop-refs (vop)
@@ -704,7 +691,7 @@
 	       ((:environment :debug-environment) (incf environment))
 	       (t (incf global)))
 	     (do ((conf (tn-global-conflicts tn)
-			(global-conflicts-tn-next conf)))
+			(global-conflicts-next-tnwise conf)))
 		 ((null conf))
 	       (incf confs)))
 	    (t
@@ -755,7 +742,7 @@
 			    (component-info component)))
 	  (barf "~S not in COMPONENT-TNs for ~S" tn component)))
        (conf
-	(do ((conf conf (global-conflicts-tn-next conf))
+	(do ((conf conf (global-conflicts-next-tnwise conf))
 	     (prev nil conf))
 	    ((null conf))
 	  (unless (eq (global-conflicts-tn conf) tn)
@@ -798,7 +785,7 @@
 (defun check-block-conflicts (component)
   (do-ir2-blocks (block component)
     (do ((conf (ir2-block-global-tns block)
-	       (global-conflicts-next conf))
+	       (global-conflicts-next-blockwise conf))
 	 (prev nil conf))
 	((null conf))
       (when prev
@@ -806,7 +793,7 @@
 		   (tn-number (global-conflicts-tn prev)))
 	  (barf "~S and ~S out of order in ~S" prev conf block)))
 
-      (unless (find-in #'global-conflicts-tn-next
+      (unless (find-in #'global-conflicts-next-tnwise
 		       conf
 		       (tn-global-conflicts
 			(global-conflicts-tn conf)))
@@ -835,7 +822,7 @@
 	   (fp (ir2-physenv-old-fp 2env))
 	   (2block (block-info (lambda-block (physenv-lambda env)))))
       (do ((conf (ir2-block-global-tns 2block)
-		 (global-conflicts-next conf)))
+		 (global-conflicts-next-blockwise conf)))
 	  ((null conf))
 	(let ((tn (global-conflicts-tn conf)))
 	  (unless (or (eq (global-conflicts-kind conf) :write)
@@ -892,11 +879,11 @@
 ;;;     keep garbage from being collected.
 (macrolet ((def (counter vto vfrom fto ffrom)
 	     `(progn
+		(declaim (type hash-table ,vto ,vfrom))
 		(defvar ,vto (make-hash-table :test 'eq))
 		(defvar ,vfrom (make-hash-table :test 'eql))
-		(proclaim '(hash-table ,vto ,vfrom))
+		(declaim (type fixnum ,counter))
 		(defvar ,counter 0)
-		(proclaim '(fixnum ,counter))
 
 		(defun ,fto (x)
 		  (or (gethash x ,vto)
@@ -923,18 +910,18 @@
      (format stream "~S ~S" (type-of leaf) (functional-debug-name leaf)))))
 
 ;;; Attempt to find a block given some thing that has to do with it.
-(declaim (ftype (function (t) cblock) block-or-lose))
+(declaim (ftype (sfunction (t) cblock) block-or-lose))
 (defun block-or-lose (thing)
   (ctypecase thing
     (cblock thing)
     (ir2-block (ir2-block-block thing))
     (vop (block-or-lose (vop-block thing)))
     (tn-ref (block-or-lose (tn-ref-vop thing)))
-    (continuation (continuation-block thing))
+    (ctran (ctran-block thing))
     (node (node-block thing))
     (component (component-head thing))
 #|    (cloop (loop-head thing))|#
-    (integer (continuation-block (num-cont thing)))
+    (integer (ctran-block (num-cont thing)))
     (functional (lambda-block (main-entry thing)))
     (null (error "Bad thing: ~S." thing))
     (symbol (block-or-lose (gethash thing *free-funs*)))))
@@ -945,60 +932,106 @@
   (format t " c~D" (cont-num cont))
   (values))
 
+(defun print-ctran (cont)
+  (declare (type ctran cont))
+  (format t "c~D " (cont-num cont))
+  (values))
+(defun print-lvar (cont)
+  (declare (type lvar cont))
+  (format t "v~D " (cont-num cont))
+  (values))
+
+(defun print-lvar-stack (stack &optional (stream *standard-output*))
+  (loop for (lvar . rest) on stack
+        do (format stream "~:[u~;d~]v~D~@[ ~]"
+                   (lvar-dynamic-extent lvar) (cont-num lvar) rest)))
+
 ;;; Print out the nodes in BLOCK in a format oriented toward
 ;;; representing what the code does.
 (defun print-nodes (block)
   (setq block (block-or-lose block))
-  (format t "~%block start c~D" (cont-num (block-start block)))
+  (pprint-logical-block (nil nil)
+    (format t "~:@_IR1 block ~D start c~D"
+            (block-number block) (cont-num (block-start block)))
+    (when (block-delete-p block)
+      (format t " <deleted>"))
 
-  (let ((last (block-last block)))
-    (terpri)
-    (do ((cont (block-start block) (node-cont (continuation-next cont))))
-	(())
-      (let ((node (continuation-next cont)))
-	(format t "~3D: " (cont-num (node-cont node)))
-	(etypecase node
-	  (ref (print-leaf (ref-leaf node)))
-	  (basic-combination
-	   (let ((kind (basic-combination-kind node)))
-	     (format t "~(~A ~A~) c~D"
-		     (if (fun-info-p kind) "known" kind)
-		     (type-of node)
-		     (cont-num (basic-combination-fun node)))
-	     (dolist (arg (basic-combination-args node))
-	       (if arg
-		   (print-continuation arg)
-		   (format t " <none>")))))
-	  (cset
-	   (write-string "set ")
-	   (print-leaf (set-var node))
-	   (print-continuation (set-value node)))
-	  (cif
-	   (format t "if c~D" (cont-num (if-test node)))
-	   (print-continuation (block-start (if-consequent node)))
-	   (print-continuation (block-start (if-alternative node))))
-	  (bind
-	   (write-string "bind ")
-	   (print-leaf (bind-lambda node)))
-	  (creturn
-	   (format t "return c~D " (cont-num (return-result node)))
-	   (print-leaf (return-lambda node)))
-	  (entry
-	   (format t "entry ~S" (entry-exits node)))
-	  (exit
-	   (let ((value (exit-value node)))
-	     (cond (value
-		    (format t "exit c~D" (cont-num value)))
-		   ((exit-entry node)
-		    (format t "exit <no value>"))
-		   (t
-		    (format t "exit <degenerate>"))))))
-	(terpri)
-	(when (eq node last) (return)))))
+    (pprint-newline :mandatory)
+    (awhen (block-info block)
+      (format t "start stack: ")
+      (print-lvar-stack (ir2-block-start-stack it))
+      (pprint-newline :mandatory))
+    (do ((ctran (block-start block) (node-next (ctran-next ctran))))
+        ((not ctran))
+      (let ((node (ctran-next ctran)))
+        (format t "~3D>~:[    ~;~:*~3D:~] "
+                (cont-num ctran)
+                (when (and (valued-node-p node) (node-lvar node))
+                  (cont-num (node-lvar node))))
+        (etypecase node
+          (ref (print-leaf (ref-leaf node)))
+          (basic-combination
+           (let ((kind (basic-combination-kind node)))
+             (format t "~(~A~A ~A~) "
+                     (if (node-tail-p node) "tail " "")
+                     kind
+                     (type-of node))
+             (print-lvar (basic-combination-fun node))
+             (dolist (arg (basic-combination-args node))
+               (if arg
+                   (print-lvar arg)
+                   (format t "<none> ")))))
+          (cset
+           (write-string "set ")
+           (print-leaf (set-var node))
+           (write-char #\space)
+           (print-lvar (set-value node)))
+          (cif
+           (write-string "if ")
+           (print-lvar (if-test node))
+           (print-ctran (block-start (if-consequent node)))
+           (print-ctran (block-start (if-alternative node))))
+          (bind
+           (write-string "bind ")
+           (print-leaf (bind-lambda node))
+           (when (functional-kind (bind-lambda node))
+             (format t " ~S ~S" :kind (functional-kind (bind-lambda node)))))
+          (creturn
+           (write-string "return ")
+           (print-lvar (return-result node))
+           (print-leaf (return-lambda node)))
+          (entry
+           (let ((cleanup (entry-cleanup node)))
+             (case (cleanup-kind cleanup)
+               ((:dynamic-extent)
+                (format t "entry DX~{ v~D~}"
+                        (mapcar #'cont-num (cleanup-info cleanup))))
+               (t
+                (format t "entry ~S" (entry-exits node))))))
+          (exit
+           (let ((value (exit-value node)))
+             (cond (value
+                    (format t "exit ")
+                    (print-lvar value))
+                   ((exit-entry node)
+                    (format t "exit <no value>"))
+                   (t
+                    (format t "exit <degenerate>")))))
+          (cast
+           (let ((value (cast-value node)))
+             (format t "cast v~D ~A[~S -> ~S]" (cont-num value)
+                     (if (cast-%type-check node) #\+ #\-)
+                     (cast-type-to-check node)
+                     (cast-asserted-type node)))))
+        (pprint-newline :mandatory)))
 
-  (let ((succ (block-succ block)))
-    (format t "successors~{ c~D~}~%"
-	    (mapcar (lambda (x) (cont-num (block-start x))) succ)))
+    (awhen (block-info block)
+      (format t "end stack: ")
+      (print-lvar-stack (ir2-block-end-stack it))
+      (pprint-newline :mandatory))
+    (let ((succ (block-succ block)))
+      (format t "successors~{ c~D~}~%"
+              (mapcar (lambda (x) (cont-num (block-start x))) succ))))
   (values))
 
 ;;; Print the guts of a TN. (logic shared between PRINT-OBJECT (TN T)
@@ -1050,27 +1083,29 @@
     (when (vop-results vop)
       (princ "=> ")
       (print-operands (vop-results vop))))
-  (terpri))
+  (pprint-newline :mandatory))
 
 ;;; Print the VOPs in the specified IR2 block.
 (defun print-ir2-block (block)
   (declare (type ir2-block block))
-  (cond
-   ((eq (block-info (ir2-block-block block)) block)
-    (format t "~%IR2 block start c~D~%"
-	    (cont-num (block-start (ir2-block-block block))))
-    (let ((label (ir2-block-%label block)))
-      (when label
-	(format t "L~D:~%" (label-id label)))))
-   (t
-    (format t "<overflow>~%")))
+  (pprint-logical-block (*standard-output* nil)
+    (cond
+      ((eq (block-info (ir2-block-block block)) block)
+       (format t "~:@_IR2 block ~D start c~D~:@_"
+               (ir2-block-number block)
+               (cont-num (block-start (ir2-block-block block))))
+       (let ((label (ir2-block-%label block)))
+         (when label
+           (format t "L~D:~:@_" (label-id label)))))
+      (t
+       (format t "<overflow>~:@_")))
 
-  (do ((vop (ir2-block-start-vop block)
-	    (vop-next vop))
-       (number 0 (1+ number)))
-      ((null vop))
-    (format t "~W: " number)
-    (print-vop vop)))
+    (do ((vop (ir2-block-start-vop block)
+              (vop-next vop))
+         (number 0 (1+ number)))
+        ((null vop))
+      (format t "~W: " number)
+      (print-vop vop))))
 
 ;;; This is like PRINT-NODES, but dumps the IR2 representation of the
 ;;; code in BLOCK.
@@ -1084,9 +1119,24 @@
   (values))
 
 ;;; Scan the IR2 blocks in emission order.
-(defun print-ir2-blocks (thing)
-  (do-ir2-blocks (block (block-component (block-or-lose thing)))
-    (print-ir2-block block))
+(defun print-ir2-blocks (thing &optional full)
+  (let* ((block (component-head (block-component (block-or-lose thing))))
+         (2block (block-info block)))
+    (pprint-logical-block (nil nil)
+      (loop while 2block
+         do (setq block (ir2-block-block 2block))
+         do (pprint-logical-block (*standard-output* nil)
+              (if full
+                  (print-nodes block)
+                  (format t "IR1 block ~D start c~D"
+                          (block-number block)
+                          (cont-num (block-start block))))
+              (pprint-indent :block 4)
+              (pprint-newline :mandatory)
+              (loop while (and 2block (eq (ir2-block-block 2block) block))
+                 do (print-ir2-block 2block)
+                 do (setq 2block (ir2-block-next 2block))))
+         do (pprint-newline :mandatory))))
   (values))
 
 ;;; Do a PRINT-NODES on BLOCK and all blocks reachable from it by
@@ -1115,12 +1165,12 @@
 
 (defvar *list-conflicts-table* (make-hash-table :test 'eq))
 
-;;; Add all ALWAYS-LIVE TNs in Block to the conflicts. TN is ignored when
-;;; it appears in the global conflicts.
+;;; Add all ALWAYS-LIVE TNs in BLOCK to the conflicts. TN is ignored
+;;; when it appears in the global conflicts.
 (defun add-always-live-tns (block tn)
   (declare (type ir2-block block) (type tn tn))
   (do ((conf (ir2-block-global-tns block)
-	     (global-conflicts-next conf)))
+	     (global-conflicts-next-blockwise conf)))
       ((null conf))
     (when (eq (global-conflicts-kind conf) :live)
       (let ((btn (global-conflicts-tn conf)))
@@ -1128,7 +1178,7 @@
 	  (setf (gethash btn *list-conflicts-table*) t)))))
   (values))
 
-;;; Add all local TNs in block to the conflicts.
+;;; Add all local TNs in BLOCK to the conflicts.
 (defun add-all-local-tns (block)
   (declare (type ir2-block block))
   (let ((ltns (ir2-block-local-tns block)))
@@ -1154,8 +1204,12 @@
   (let ((confs (tn-global-conflicts tn)))
     (cond (confs
 	   (clrhash *list-conflicts-table*)
-	   (do ((conf confs (global-conflicts-tn-next conf)))
+	   (do ((conf confs (global-conflicts-next-tnwise conf)))
 	       ((null conf))
+             (format t "~&#<block ~D kind ~S>~%"
+                     (block-number (ir2-block-block (global-conflicts-block
+						     conf)))
+                     (global-conflicts-kind conf))
 	     (let ((block (global-conflicts-block conf)))
 	       (add-always-live-tns block tn)
 	       (if (eq (global-conflicts-kind conf) :live)
@@ -1179,7 +1233,7 @@
 				(not (tn-global-conflicts tn)))
 		       (res tn)))))
 	       (do ((gtn (ir2-block-global-tns block)
-			 (global-conflicts-next gtn)))
+			 (global-conflicts-next-blockwise gtn)))
 		   ((null gtn))
 		 (when (or (eq (global-conflicts-kind gtn) :live)
 			   (/= (sbit confs (global-conflicts-number gtn)) 0))
@@ -1188,7 +1242,7 @@
 
 (defun nth-vop (thing n)
   #!+sb-doc
-  "Return the Nth VOP in the IR2-Block pointed to by Thing."
+  "Return the Nth VOP in the IR2-BLOCK pointed to by THING."
   (let ((block (block-info (block-or-lose thing))))
     (do ((i 0 (1+ i))
 	 (vop (ir2-block-start-vop block) (vop-next vop)))
