@@ -64,6 +64,15 @@
 	 (nil #:mu 0 simple-array-nil
 	      :complex-typecode #.sb!vm:complex-vector-nil-widetag
 	      :importance 0)
+         #!-sb-unicode
+	 (character ,(code-char 0) 8 simple-base-string
+		    ;; (SIMPLE-BASE-STRINGs are stored with an extra
+		    ;; trailing #\NULL for convenience in calling out
+		    ;; to C.)
+		    :n-pad-elements 1
+	            :complex-typecode #.sb!vm:complex-base-string-widetag
+	            :importance 17)
+         #!+sb-unicode
 	 (base-char ,(code-char 0) 8 simple-base-string
 		    ;; (SIMPLE-BASE-STRINGs are stored with an extra
 		    ;; trailing #\NULL for convenience in calling out
@@ -71,6 +80,11 @@
 		    :n-pad-elements 1
 	            :complex-typecode #.sb!vm:complex-base-string-widetag
 	            :importance 17)
+         #!+sb-unicode
+	 (character ,(code-char 0) 32 simple-character-string
+		    :n-pad-elements 1
+	            :complex-typecode #.sb!vm:complex-character-string-widetag
+	            :importance 17)
 	 (single-float 0.0f0 32 simple-array-single-float
 	  :importance 6)
 	 (double-float 0.0d0 64 simple-array-double-float
