@@ -32,11 +32,7 @@
 (declaim (special *universal-type*))
 
 ;;; This is sorta semantically equivalent to SXHASH, but optimized for
-;;; legal function names. Note: semantically equivalent does *not*
-;;; mean that it always returns the same value as SXHASH, just that it
-;;; satisfies the formal definition of SXHASH. The ``sorta'' is
-;;; because SYMBOL-HASH will not necessarily return the same value in
-;;; different lisp images.
+;;; legal function names.
 ;;;
 ;;; Why optimize? We want to avoid the fully-general TYPECASE in ordinary
 ;;; SXHASH, because
@@ -58,17 +54,17 @@
 ;;; aren't used too early in cold boot for SXHASH to run).
 #!-sb-fluid (declaim (inline globaldb-sxhashoid))
 (defun globaldb-sxhashoid (x)
-  (cond #-sb-xc-host ; (SYMBOL-HASH doesn't exist on cross-compilation host.)
-	((symbolp x)
-	 (symbol-hash x))
-	#-sb-xc-host ; (SYMBOL-HASH doesn't exist on cross-compilation host.)
+  (cond	((symbolp x) (sxhash x))
 	((and (listp x)
 	      (eq (first x) 'setf)
 	      (let ((rest (rest x)))
 		(and (symbolp (car rest))
 		     (null (cdr rest)))))
-	 (logxor (symbol-hash (second x))
-		 110680597))
+	 ;; We need to declare the type of the value we're feeding to
+	 ;; SXHASH so that the DEFTRANSFORM on symbols kicks in.
+	 (let ((symbol (second x)))
+	   (declare (symbol symbol))
+	   (logxor (sxhash symbol) 110680597)))
 	(t (sxhash x))))
 
 ;;; Given any non-negative integer, return a prime number >= to it.
@@ -100,7 +96,7 @@
 ;;; At run time, we represent the type of info that we want by a small
 ;;; non-negative integer.
 (eval-when (:compile-toplevel :load-toplevel :execute)
-  (defconstant type-number-bits 6))
+  (def!constant type-number-bits 6))
 (deftype type-number () `(unsigned-byte ,type-number-bits))
 
 ;;; Why do we suppress the :COMPILE-TOPLEVEL situation here when we're
@@ -263,7 +259,7 @@
 ;;; order in which the TYPE-INFO-creation forms are generated doesn't
 ;;; match the relative order in which the forms need to be executed at
 ;;; cold load time.
-(defparameter *reversed-type-info-init-forms* nil)
+(defparameter *!reversed-type-info-init-forms* nil)
 
 ;;; Define a new type of global information for CLASS. TYPE is the
 ;;; name of the type, DEFAULT is the value for that type when it
@@ -320,7 +316,7 @@
 			 (declare (ignorable name))
 			 ,',default))
 		(setf (type-info-type type-info) ',',type-spec))
-	     *reversed-type-info-init-forms*))
+	     *!reversed-type-info-init-forms*))
      ',type))
 
 ) ; EVAL-WHEN
@@ -360,7 +356,7 @@
 	     ,(do-compact-info name class type type-number value
 			       n-env body)))))
 
-(eval-when (:compile-toplevel :load-toplevel :execute)
+(eval-when (#-sb-xc :compile-toplevel :load-toplevel :execute)
 
 ;;; Return code to iterate over a compact info environment.
 (defun do-compact-info (name-var class-var type-var type-number-var value-var
@@ -480,7 +476,7 @@
 ;;;; compact info environments
 
 ;;; The upper limit on the size of the ENTRIES vector in a COMPACT-INFO-ENV.
-(defconstant compact-info-env-entries-bits 16)
+(def!constant compact-info-env-entries-bits 16)
 (deftype compact-info-entries-index () `(unsigned-byte ,compact-info-env-entries-bits))
 
 ;;; the type of the values in COMPACT-INFO-ENTRIES-INFO
@@ -516,8 +512,8 @@
   ;; last entry.
   (entries-info (missing-arg) :type (simple-array compact-info-entry (*))))
 
-(defconstant compact-info-entry-type-mask (ldb (byte type-number-bits 0) -1))
-(defconstant compact-info-entry-last (ash 1 type-number-bits))
+(def!constant compact-info-entry-type-mask (ldb (byte type-number-bits 0) -1))
+(def!constant compact-info-entry-last (ash 1 type-number-bits))
 
 ;;; Return the value of the type corresponding to NUMBER for the
 ;;; currently cached name in ENV.
@@ -541,7 +537,20 @@
 ;;; Encache NAME in the compact environment ENV. HASH is the
 ;;; GLOBALDB-SXHASHOID of NAME.
 (defun compact-info-lookup (env name hash)
-  (declare (type compact-info-env env) (type index hash))
+  (declare (type compact-info-env env)
+	   ;; FIXME: this used to read (TYPE INDEX HASH), but that was
+	   ;; wrong, because HASH was a positive fixnum, not a (MOD
+	   ;; MOST-POSITIVE-FIXNUM).
+	   ;;
+	   ;; However, this, its replacement, is also wrong.  In the
+	   ;; cross-compiler, GLOBALDB-SXHASHOID is essentially
+	   ;; SXHASH.  But our host compiler could have any value at
+	   ;; all as its MOST-POSITIVE-FIXNUM, and so could in
+	   ;; principle return a value exceeding our target positive
+	   ;; fixnum range.
+	   ;;
+	   ;; My brain hurts.  -- CSR, 2003-08-28
+	   (type (integer 0 #.sb!xc:most-positive-fixnum) hash))
   (let* ((table (compact-info-env-table env))
 	 (len (length table))
 	 (len-2 (- len 2))
@@ -572,7 +581,7 @@
 
 ;;; the exact density (modulo rounding) of the hashtable in a compact
 ;;; info environment in names/bucket
-(defconstant compact-info-environment-density 65)
+(def!constant compact-info-environment-density 65)
 
 ;;; Return a new compact info environment that holds the same
 ;;; information as ENV.
@@ -703,7 +712,9 @@
 
 ;;; Just like COMPACT-INFO-LOOKUP, only do it on a volatile environment.
 (defun volatile-info-lookup (env name hash)
-  (declare (type volatile-info-env env) (type index hash))
+  (declare (type volatile-info-env env)
+	   ;; FIXME: see comment in COMPACT-INFO-LOOKUP
+	   (type (integer 0 #.sb!xc:most-positive-fixnum) hash))
   (let ((table (volatile-info-env-table env)))
     (macrolet ((lookup (test)
 		 `(dolist (entry (svref table (mod hash (length table))) ())
@@ -714,11 +725,10 @@
 		(lookup eq)
 		(lookup equal)))
       (setf (volatile-info-env-cache-name env) name)))
-
   (values))
 
-;;; Given a volatile environment Env, bind Table-Var the environment's table
-;;; and Index-Var to the index of Name's bucket in the table. We also flush
+;;; Given a volatile environment ENV, bind TABLE-VAR the environment's table
+;;; and INDEX-VAR to the index of NAME's bucket in the table. We also flush
 ;;; the cache so that things will be consistent if body modifies something.
 (eval-when (:compile-toplevel :execute)
   (#+sb-xc-host cl:defmacro
@@ -819,15 +829,14 @@
   ;; Constant CLASS and TYPE is an overwhelmingly common special case,
   ;; and we can implement it much more efficiently than the general case.
   (if (and (constantp class) (constantp type))
-      (let ((info (type-info-or-lose class type))
-	    (value (gensym "VALUE"))
-	    (foundp (gensym "FOUNDP")))
-	`(multiple-value-bind (,value ,foundp)
-	     (get-info-value ,name
-			     ,(type-info-number info)
-			     ,@(when env-list-p `(,env-list))) 
-	   (declare (type ,(type-info-type info) ,value))
-	   (values ,value ,foundp)))
+      (let ((info (type-info-or-lose class type)))
+	(with-unique-names (value foundp)
+	  `(multiple-value-bind (,value ,foundp)
+	       (get-info-value ,name
+			       ,(type-info-number info)
+			       ,@(when env-list-p `(,env-list))) 
+	     (declare (type ,(type-info-type info) ,value))
+	     (values ,value ,foundp))))
       whole))
 (defun (setf info) (new-value
 		    class
@@ -837,13 +846,13 @@
   (let* ((info (type-info-or-lose class type))
 	 (tin (type-info-number info)))
     (if env-list-p
-      (set-info-value name
-		      tin
-		      new-value
-		      (get-write-info-env env-list))
-      (set-info-value name
-		      tin
-		      new-value)))
+	(set-info-value name
+			tin
+			new-value
+			(get-write-info-env env-list))
+	(set-info-value name
+			tin
+			new-value)))
   new-value)
 ;;; FIXME: We'd like to do this, but Python doesn't support
 ;;; compiler macros and it's hard to change it so that it does.
@@ -882,7 +891,7 @@
 ;;;
 ;;; FIXME: actually seems to be measured in percent, should be
 ;;; converted to be measured in names/bucket
-(defconstant volatile-info-environment-density 50)
+(def!constant volatile-info-environment-density 50)
 
 ;;; Make a new volatile environment of the specified size.
 (defun make-info-environment (&key (size 42) (name "Unknown"))
@@ -897,7 +906,6 @@
 ;;; the current environment, allowing any inherited info to become
 ;;; visible. We return true if there was any info.
 (defun clear-info (class type name)
-  #!+sb-doc
   (let ((info (type-info-or-lose class type)))
     (clear-info-value name (type-info-number info))))
 #!-sb-fluid
@@ -1036,7 +1044,7 @@
   ;;   (or approximate-fun-type null)).
   ;; It was changed to T as a hopefully-temporary hack while getting
   ;; cold init problems untangled.
-  :type-spec t) 
+  :type-spec t)
 
 ;;; where this information came from:
 ;;;    :ASSUMED  = from uses of the object
@@ -1119,13 +1127,6 @@
   :type :ir1-convert
   :type-spec (or function null))
 
-;;; a function which gets a chance to do stuff to the IR1 for any call
-;;; to this function.
-(define-info-type
-  :class :function
-  :type :ir1-transform
-  :type-spec (or function null))
-
 ;;; If a function is "known" to the compiler, then this is a FUN-INFO
 ;;; structure containing the info used to special-case compilation.
 (define-info-type
@@ -1143,7 +1144,7 @@
 (define-info-type
   :class :function
   :type :definition
-  :type-spec t
+  :type-spec (or fdefn null)
   :default nil)
 
 ;;;; definitions for other miscellaneous information
@@ -1154,7 +1155,7 @@
 (define-info-type
   :class :variable
   :type :kind
-  :type-spec (member :special :constant :global :alien)
+  :type-spec (member :special :constant :macro :global :alien)
   :default (if (symbol-self-evaluating-p name)
 	       :constant
 	       :global))
@@ -1189,6 +1190,13 @@
 	       name
 	       (bug "constant lookup of nonconstant ~S" name)))
 
+;;; the macro-expansion for symbol-macros
+(define-info-type
+  :class :variable
+  :type :macro-expansion
+  :type-spec t
+  :default nil)
+
 (define-info-type
   :class :variable
   :type :alien-info
@@ -1252,8 +1260,8 @@
 ;;; meaningful error if we only have the cons.
 (define-info-type
   :class :type
-  :type :class
-  :type-spec (or sb!kernel::class-cell null)
+  :type :classoid
+  :type-spec (or sb!kernel::classoid-cell null)
   :default nil)
 
 ;;; layout for this type being used by the compiler
@@ -1261,8 +1269,8 @@
   :class :type
   :type :compiler-layout
   :type-spec (or layout null)
-  :default (let ((class (sb!xc:find-class name nil)))
-	     (when class (class-layout class))))
+  :default (let ((class (find-classoid name nil)))
+	     (when class (classoid-layout class))))
 
 (define-info-class :typed-structure)
 (define-info-type
@@ -1387,7 +1395,7 @@
 ;;; we can set their DEFAULT and TYPE slots.
 (macrolet ((frob ()
 	     `(!cold-init-forms
-		,@(reverse *reversed-type-info-init-forms*))))
+		,@(reverse *!reversed-type-info-init-forms*))))
   (frob))
 
 ;;;; a hack for detecting
