@@ -109,11 +109,14 @@
   (ctran-starts-block next)
   (let* ((found (or (lexenv-find name blocks)
 		    (compiler-error "return for unknown block: ~S" name)))
+         (exit-ctran (second found))
 	 (value-ctran (make-ctran))
          (value-lvar (make-lvar))
 	 (entry (first found))
 	 (exit (make-exit :entry entry
 			  :value value-lvar)))
+    (when (ctran-deleted-p exit-ctran)
+      (throw 'locall-already-let-converted exit-ctran))
     (push exit (entry-exits entry))
     (setf (lvar-dest value-lvar) exit)
     (ir1-convert start value-ctran value-lvar value)
@@ -121,7 +124,7 @@
     (let ((home-lambda (ctran-home-lambda-or-null start)))
       (when home-lambda
 	(push entry (lambda-calls-or-closes home-lambda))))
-    (use-continuation exit (second found) (third found))))
+    (use-continuation exit exit-ctran (third found))))
 
 ;;; Return a list of the segments of a TAGBODY. Each segment looks
 ;;; like (<tag> <form>* (go <next tag>)). That is, we break up the
@@ -399,7 +402,7 @@
     (if (template-more-args-type template)
 	(when (< nargs min)
 	  (bug "Primitive ~A was called with ~R argument~:P, ~
-	    	but wants at least ~R."
+                but wants at least ~R."
 	       name
 	       nargs
 	       min))
@@ -543,23 +546,26 @@
   During evaluation of the Forms, bind the Vars to the result of evaluating the
   Value forms. The variables are bound in parallel after all of the Values are
   evaluated."
-  (if (null bindings)
-      (ir1-translate-locally body start next result)
-      (multiple-value-bind (forms decls)
-          (parse-body body :doc-string-allowed nil)
-        (multiple-value-bind (vars values) (extract-let-vars bindings 'let)
-          (binding* ((ctran (make-ctran))
-                     (fun-lvar (make-lvar))
-                     ((next result)
-                      (processing-decls (decls vars nil next result)
-                        (let ((fun (ir1-convert-lambda-body
-                                    forms
-                                    vars
-                                    :debug-name (debug-namify "LET S"
-                                                              bindings))))
-                          (reference-leaf start ctran fun-lvar fun))
-                        (values next result))))
-            (ir1-convert-combination-args fun-lvar ctran next result values))))))
+  (cond ((null bindings)
+         (ir1-translate-locally body start next result))
+        ((listp bindings)
+         (multiple-value-bind (forms decls)
+             (parse-body body :doc-string-allowed nil)
+           (multiple-value-bind (vars values) (extract-let-vars bindings 'let)
+             (binding* ((ctran (make-ctran))
+                        (fun-lvar (make-lvar))
+                        ((next result)
+                         (processing-decls (decls vars nil next result)
+                                           (let ((fun (ir1-convert-lambda-body
+                                                       forms
+                                                       vars
+                                                       :debug-name (debug-namify "LET S"
+                                                                                 bindings))))
+                                             (reference-leaf start ctran fun-lvar fun))
+                                           (values next result))))
+                       (ir1-convert-combination-args fun-lvar ctran next result values)))))
+        (t
+         (compiler-error "Malformed LET bindings: ~S." bindings))))
 
 (def-ir1-translator let* ((bindings &body body)
 			  start next result)
@@ -567,17 +573,19 @@
   "LET* ({(Var [Value]) | Var}*) Declaration* Form*
   Similar to LET, but the variables are bound sequentially, allowing each Value
   form to reference any of the previous Vars."
-  (multiple-value-bind (forms decls)
-      (parse-body body :doc-string-allowed nil)
-    (multiple-value-bind (vars values) (extract-let-vars bindings 'let*)
-      (processing-decls (decls vars nil start next)
-        (ir1-convert-aux-bindings start 
-                                  next 
-                                  result
-                                  forms
-                                  vars 
-                                  values)))))
-
+  (if (listp bindings)
+      (multiple-value-bind (forms decls)
+          (parse-body body :doc-string-allowed nil)
+        (multiple-value-bind (vars values) (extract-let-vars bindings 'let*)
+          (processing-decls (decls vars nil start next)
+                            (ir1-convert-aux-bindings start 
+                                                      next 
+                                                      result
+                                                      forms
+                                                      vars 
+                                                      values))))
+      (compiler-error "Malformed LET* bindings: ~S." bindings)))
+  
 ;;; logic shared between IR1 translators for LOCALLY, MACROLET,
 ;;; and SYMBOL-MACROLET
 ;;;
