@@ -239,6 +239,7 @@
 	       ;; can't contain other objects
 	       (unless (typep value
 			      '(or #-sb-xc-host unboxed-array
+				   #+sb-xc-host (simple-array (unsigned-byte 8) (*))
 				   symbol
 				   number
 				   character
@@ -575,7 +576,14 @@
                                  :notinline))
                         (let ((functional (defined-fun-functional leaf)))
                           (when (and functional
-                                     (not (functional-kind functional)))
+                                     (not (functional-kind functional))
+                                     ;; Bug MISC.320: ir1-transform
+                                     ;; can create a reference to a
+                                     ;; inline-expanded function,
+                                     ;; defined in another component.
+                                     (not (and (lambda-p functional)
+                                               (neq (lambda-component functional)
+                                                    *current-component*))))
                             (maybe-reanalyze-functional functional))))
                    (when (and (lambda-p leaf)
                               (memq (functional-kind leaf)
@@ -1067,7 +1075,7 @@
 		(functional
 		 (when (policy *lexenv* (>= speed inhibit-warnings))
 		   (compiler-notify "ignoring ~A declaration not at ~
-				     definition of local function:~%  ~S"
+                                     definition of local function:~%  ~S"
 				    sense name)))
 		(global-var
 		 (push (cons name (make-new-inlinep found sense))
