@@ -120,12 +120,12 @@
   is intended to be wrapped around the compilation of all files in the same
   system. These keywords are defined:
     :OVERRIDE Boolean-Form
-	One of the effects of this form is to delay undefined warnings
-	until the end of the form, instead of giving them at the end of each
-	compilation. If OVERRIDE is NIL (the default), then the outermost
-	WITH-COMPILATION-UNIT form grabs the undefined warnings. Specifying
-	OVERRIDE true causes that form to grab any enclosed warnings, even if
-	it is enclosed by another WITH-COMPILATION-UNIT."
+        One of the effects of this form is to delay undefined warnings
+        until the end of the form, instead of giving them at the end of each
+        compilation. If OVERRIDE is NIL (the default), then the outermost
+        WITH-COMPILATION-UNIT form grabs the undefined warnings. Specifying
+        OVERRIDE true causes that form to grab any enclosed warnings, even if
+        it is enclosed by another WITH-COMPILATION-UNIT."
   `(%with-compilation-unit (lambda () ,@body) ,@options))
 
 (defun %with-compilation-unit (fn &key override)
@@ -225,12 +225,12 @@
 	    (when summary
 	      (if (eq kind :variable)
 		  (compiler-warn
-		   "~:[This ~(~A~) is~;These ~(~A~)s are~] undefined:~
-		    ~%  ~{~<~%  ~1:;~S~>~^ ~}"
+                   "~:[This ~(~A~) is~;These ~(~A~)s are~] undefined:~
+                    ~%  ~{~<~%  ~1:;~S~>~^ ~}"
 		   (cdr summary) kind summary)
 		  (compiler-style-warn
-		   "~:[This ~(~A~) is~;These ~(~A~)s are~] undefined:~
-		   ~%  ~{~<~%  ~1:;~S~>~^ ~}"
+                   "~:[This ~(~A~) is~;These ~(~A~)s are~] undefined:~
+                   ~%  ~{~<~%  ~1:;~S~>~^ ~}"
 		   (cdr summary) kind summary))))))))
 
   (unless (and (not abort-p)
@@ -287,17 +287,20 @@
   (maybe-mumble "opt")
   (event ir1-optimize-until-done)
   (let ((count 0)
-	(cleared-reanalyze nil))
+	(cleared-reanalyze nil)
+        (fastp nil))
     (loop
       (when (component-reanalyze component)
 	(setq count 0)
 	(setq cleared-reanalyze t)
 	(setf (component-reanalyze component) nil))
       (setf (component-reoptimize component) nil)
-      (ir1-optimize component)
+      (ir1-optimize component fastp)
       (cond ((component-reoptimize component)
              (incf count)
-             (when (= count *max-optimize-iterations*)
+             (when (and (>= count *max-optimize-iterations*)
+                        (not (component-reanalyze component))
+                        (eq (component-reoptimize component) :maybe))
                (maybe-mumble "*")
                (cond ((retry-delayed-ir1-transforms :optimize)
 		      (maybe-mumble "+")
@@ -314,7 +317,8 @@
 	    (t
              (maybe-mumble " ")
 	     (return)))
-      (maybe-mumble "."))
+      (setq fastp (>= count *max-optimize-iterations*))
+      (maybe-mumble (if fastp "-" ".")))
     (when cleared-reanalyze
       (setf (component-reanalyze component) t)))
   (values))
