@@ -58,30 +58,32 @@
 ;;; spurious attempts at transformation (and possible repeated
 ;;; warnings.)
 (deftransform typep ((object type))
-  (unless (constant-continuation-p type)
+  (unless (constant-lvar-p type)
     (give-up-ir1-transform "can't open-code test of non-constant type"))
-  `(typep object ',(continuation-value type)))
+  `(typep object ',(lvar-value type)))
 
-;;; If the continuation OBJECT definitely is or isn't of the specified
+;;; If the lvar OBJECT definitely is or isn't of the specified
 ;;; type, then return T or NIL as appropriate. Otherwise quietly
 ;;; GIVE-UP-IR1-TRANSFORM.
 (defun ir1-transform-type-predicate (object type)
-  (declare (type continuation object) (type ctype type))
-  (let ((otype (continuation-type object)))
+  (declare (type lvar object) (type ctype type))
+  (let ((otype (lvar-type object)))
     (cond ((not (types-equal-or-intersect otype type))
 	   nil)
 	  ((csubtypep otype type)
 	   t)
+          ((eq type *empty-type*)
+           nil)
 	  (t
 	   (give-up-ir1-transform)))))
 
 ;;; Flush %TYPEP tests whose result is known at compile time.
 (deftransform %typep ((object type))
-  (unless (constant-continuation-p type)
+  (unless (constant-lvar-p type)
     (give-up-ir1-transform))
   (ir1-transform-type-predicate
    object
-   (specifier-type (continuation-value type))))
+   (ir1-transform-specifier-type (lvar-value type))))
 
 ;;; This is the IR1 transform for simple type predicates. It checks
 ;;; whether the single argument is known to (not) be of the
@@ -89,7 +91,7 @@
 (deftransform fold-type-predicate ((object) * * :node node :defun-only t)
   (let ((ctype (gethash (leaf-source-name
 			 (ref-leaf
-			  (continuation-use
+			  (lvar-uses
 			   (basic-combination-fun node))))
 			*backend-predicate-types*)))
     (aver ctype)
@@ -97,11 +99,10 @@
 
 ;;; If FIND-CLASS is called on a constant class, locate the CLASS-CELL
 ;;; at load time.
-(deftransform find-class ((name) ((constant-arg symbol)) *
-			  :when :both)
-  (let* ((name (continuation-value name))
-	 (cell (find-class-cell name)))
-    `(or (class-cell-class ',cell)
+(deftransform find-classoid ((name) ((constant-arg symbol)) *)
+  (let* ((name (lvar-value name))
+	 (cell (find-classoid-cell name)))
+    `(or (classoid-cell-classoid ',cell)
 	 (error "class not yet defined: ~S" name))))
 
 ;;;; standard type predicates, i.e. those defined in package COMMON-LISP,
@@ -147,6 +148,9 @@
 
 (define-source-transform atom (x)
   `(not (consp ,x)))
+#!+sb-unicode
+(define-source-transform base-char-p (x)
+  `(typep ,x 'base-char))
 
 ;;;; TYPEP source transform
 
@@ -154,7 +158,6 @@
 ;;; binds specified by TYPE. BASE is the name of the base type, for
 ;;; declaration. We make SAFETY locally 0 to inhibit any checking of
 ;;; this assertion.
-#!-negative-zero-is-not-zero
 (defun transform-numeric-bound-test (n-object type base)
   (declare (type numeric-type type))
   (let ((low (numeric-type-low type))
@@ -163,55 +166,12 @@
        (declare (optimize (safety 0)))
        (and ,@(when low
 		(if (consp low)
-		    `((> (the ,base ,n-object) ,(car low)))
-		    `((>= (the ,base ,n-object) ,low))))
+		    `((> (truly-the ,base ,n-object) ,(car low)))
+		    `((>= (truly-the ,base ,n-object) ,low))))
 	    ,@(when high
 		(if (consp high)
-		    `((< (the ,base ,n-object) ,(car high)))
-		    `((<= (the ,base ,n-object) ,high))))))))
-
-#!+negative-zero-is-not-zero
-(defun transform-numeric-bound-test (n-object type base)
-  (declare (type numeric-type type))
-  (let ((low (numeric-type-low type))
-	(high (numeric-type-high type))
-	(float-type-p (csubtypep type (specifier-type 'float)))
-	(x (gensym))
-	(y (gensym)))
-    `(locally
-       (declare (optimize (safety 0)))
-       (and ,@(when low
-		(if (consp low)
-		    `((let ((,x (the ,base ,n-object))
-			    (,y ,(car low)))
-			,(if (not float-type-p)
-			    `(> ,x ,y)
-			    `(if (and (zerop ,x) (zerop ,y))
-				 (> (float-sign ,x) (float-sign ,y))
-				 (> ,x ,y)))))
-		    `((let ((,x (the ,base ,n-object))
-			    (,y ,low))
-			,(if (not float-type-p)
-			    `(>= ,x ,y)
-			    `(if (and (zerop ,x) (zerop ,y))
-				 (>= (float-sign ,x) (float-sign ,y))
-				 (>= ,x ,y)))))))
-	    ,@(when high
-		(if (consp high)
-		    `((let ((,x (the ,base ,n-object))
-			    (,y ,(car high)))
-			,(if (not float-type-p)
-			     `(< ,x ,y)
-			     `(if (and (zerop ,x) (zerop ,y))
-				  (< (float-sign ,x) (float-sign ,y))
-				  (< ,x ,y)))))
-		    `((let ((,x (the ,base ,n-object))
-			    (,y ,high))
-			,(if (not float-type-p)
-			     `(<= ,x ,y)
-			     `(if (and (zerop ,x) (zerop ,y))
-				  (<= (float-sign ,x) (float-sign ,y))
-				  (<= ,x ,y)))))))))))
+		    `((< (truly-the ,base ,n-object) ,(car high)))
+		    `((<= (truly-the ,base ,n-object) ,high))))))))
 
 ;;; Do source transformation of a test of a known numeric type. We can
 ;;; assume that the type doesn't have a corresponding predicate, since
@@ -232,7 +192,11 @@
 (defun source-transform-numeric-typep (object type)
   (let* ((class (numeric-type-class type))
 	 (base (ecase class
-		 (integer (containing-integer-type type))
+		 (integer (containing-integer-type
+                           (if (numeric-type-complexp type)
+                               (modified-numeric-type type
+                                                      :complexp :real)
+                               type)))
 		 (rational 'rational)
 		 (float (or (numeric-type-format type) 'float))
 		 ((nil) 'real))))
@@ -243,8 +207,8 @@
 	       ,(transform-numeric-bound-test n-object type base)))
 	(:complex
 	 `(and (complexp ,n-object)
-	       ,(once-only ((n-real `(realpart (the complex ,n-object)))
-			    (n-imag `(imagpart (the complex ,n-object))))
+	       ,(once-only ((n-real `(realpart (truly-the complex ,n-object)))
+			    (n-imag `(imagpart (truly-the complex ,n-object))))
 		  `(progn
 		     ,n-imag ; ignorable
 		     (and (typep ,n-real ',base)
@@ -263,8 +227,8 @@
   (let ((spec (hairy-type-specifier type)))
     (cond ((unknown-type-p type)
 	   (when (policy *lexenv* (> speed inhibit-warnings))
-	     (compiler-note "can't open-code test of unknown type ~S"
-			    (type-specifier type)))
+	     (compiler-notify "can't open-code test of unknown type ~S"
+			      (type-specifier type)))
 	   `(%typep ,object ',spec))
 	  (t
 	   (ecase (first spec)
@@ -275,25 +239,33 @@
 					    `(typep ,n-obj ',x))
 					  (rest spec))))))))))
 
+(defun source-transform-negation-typep (object type)
+  (declare (type negation-type type))
+  (let ((spec (type-specifier (negation-type-type type))))
+    `(not (typep ,object ',spec))))
+
 ;;; Do source transformation for TYPEP of a known union type. If a
 ;;; union type contains LIST, then we pull that out and make it into a
-;;; single LISTP call. Note that if SYMBOL is in the union, then LIST
-;;; will be a subtype even without there being any (member NIL). We
-;;; just drop through to the general code in this case, rather than
-;;; trying to optimize it.
+;;; single LISTP call.  Note that if SYMBOL is in the union, then LIST
+;;; will be a subtype even without there being any (member NIL).  We
+;;; currently just drop through to the general code in this case,
+;;; rather than trying to optimize it (but FIXME CSR 2004-04-05: it
+;;; wouldn't be hard to optimize it after all).
 (defun source-transform-union-typep (object type)
   (let* ((types (union-type-types type))
-	 (ltype (specifier-type 'list))
-	 (mtype (find-if #'member-type-p types)))
-    (if (and mtype (csubtypep ltype type))
-	(let ((members (member-type-members mtype)))
-	  (once-only ((n-obj object))
-	    `(or (listp ,n-obj)
-		 (typep ,n-obj
-			'(or ,@(mapcar #'type-specifier
-				       (remove (specifier-type 'cons)
-					       (remove mtype types)))
-			     (member ,@(remove nil members)))))))
+         (type-cons (specifier-type 'cons))
+	 (mtype (find-if #'member-type-p types))
+         (members (when mtype (member-type-members mtype))))
+    (if (and mtype
+             (memq nil members)
+             (memq type-cons types))
+	(once-only ((n-obj object))
+          `(or (listp ,n-obj)
+               (typep ,n-obj
+                      '(or ,@(mapcar #'type-specifier
+                                     (remove type-cons
+                                             (remove mtype types)))
+                        (member ,@(remove nil members))))))
 	(once-only ((n-obj object))
 	  `(or ,@(mapcar (lambda (x)
 			   `(typep ,n-obj ',(type-specifier x)))
@@ -310,10 +282,8 @@
 (defun source-transform-cons-typep (object type)
   (let* ((car-type (cons-type-car-type type))
 	 (cdr-type (cons-type-cdr-type type)))
-    (let ((car-test-p (not (or (type= car-type *wild-type*)
-			       (type= car-type (specifier-type t)))))
-	  (cdr-test-p (not (or (type= cdr-type *wild-type*)
-			       (type= cdr-type (specifier-type t))))))
+    (let ((car-test-p (not (type= car-type *universal-type*)))
+	  (cdr-test-p (not (type= cdr-type *universal-type*))))
       (if (and (not car-test-p) (not cdr-test-p))
 	  `(consp ,object)
 	  (once-only ((n-obj object))
@@ -325,6 +295,21 @@
 			`((typep (cdr ,n-obj)
 				 ',(type-specifier cdr-type))))))))))
  
+(defun source-transform-character-set-typep (object type)
+  (let ((pairs (character-set-type-pairs type)))
+    (if (and (= (length pairs) 1)
+            (= (caar pairs) 0)
+            (= (cdar pairs) (1- sb!xc:char-code-limit)))
+       `(characterp ,object)
+       (once-only ((n-obj object))
+         (let ((n-code (gensym "CODE")))
+           `(and (characterp ,n-obj)
+                 (let ((,n-code (sb!xc:char-code ,n-obj)))
+                   (or
+                    ,@(loop for pair in pairs
+                            collect
+                            `(<= ,(car pair) ,n-code ,(cdr pair)))))))))))
+
 ;;; Return the predicate and type from the most specific entry in
 ;;; *TYPE-PREDICATES* that is a supertype of TYPE.
 (defun find-supertype-predicate (type)
@@ -385,12 +370,12 @@
 ;;; then we also check whether the layout for the object is invalid
 ;;; and signal an error if so. Otherwise, look up the indirect
 ;;; class-cell and call CLASS-CELL-TYPEP at runtime.
-(deftransform %instance-typep ((object spec) (* *) * :node node :when :both)
-  (aver (constant-continuation-p spec))
-  (let* ((spec (continuation-value spec))
+(deftransform %instance-typep ((object spec) (* *) * :node node)
+  (aver (constant-lvar-p spec))
+  (let* ((spec (lvar-value spec))
 	 (class (specifier-type spec))
-	 (name (sb!xc:class-name class))
-	 (otype (continuation-type object))
+	 (name (classoid-name class))
+	 (otype (lvar-type object))
 	 (layout (let ((res (info :type :compiler-layout name)))
 		   (if (and res (not (layout-invalid res)))
 		       res
@@ -402,9 +387,9 @@
       ((csubtypep otype class)
        t)
       ;; If not properly named, error.
-      ((not (and name (eq (sb!xc:find-class name) class)))
+      ((not (and name (eq (find-classoid name) class)))
        (compiler-error "can't compile TYPEP of anonymous or undefined ~
-			class:~%  ~S"
+                        class:~%  ~S"
 		       class))
       (t
         ;; Delay the type transform to give type propagation a chance.
@@ -420,8 +405,8 @@
 	     (t
 	      (values '(lambda (x) (declare (ignore x)) t) 'layout-of)))
 	 (cond
-	   ((and (eq (class-state class) :sealed) layout
-		 (not (class-subclasses class)))
+	   ((and (eq (classoid-state class) :sealed) layout
+		 (not (classoid-subclasses class)))
 	    ;; Sealed and has no subclasses.
 	    (let ((n-layout (gensym)))
 	      `(and (,pred object)
@@ -430,7 +415,7 @@
 			      `((when (layout-invalid ,n-layout)
 				  (%layout-invalid-error object ',layout))))
 		      (eq ,n-layout ',layout)))))
-	   ((and (typep class 'basic-structure-class) layout)
+	   ((and (typep class 'basic-structure-classoid) layout)
 	    ;; structure type tests; hierarchical layout depths
 	    (let ((depthoid (layout-depthoid layout))
 		  (n-layout (gensym)))
@@ -468,9 +453,9 @@
 	   (t
 	    (/noshow "default case -- ,PRED and CLASS-CELL-TYPEP")
 	    `(and (,pred object)
-		  (class-cell-typep (,get-layout object)
-				    ',(find-class-cell name)
-				    object)))))))))
+		  (classoid-cell-typep (,get-layout object)
+				       ',(find-classoid-cell name)
+		                       object)))))))))
 
 ;;; If the specifier argument is a quoted constant, then we consider
 ;;; converting into a simple predicate or other stuff. If the type is
@@ -489,17 +474,23 @@
   ;; KLUDGE: It looks bad to only do this on explicitly quoted forms,
   ;; since that would overlook other kinds of constants. But it turns
   ;; out that the DEFTRANSFORM for TYPEP detects any constant
-  ;; continuation, transforms it into a quoted form, and gives this
+  ;; lvar, transforms it into a quoted form, and gives this
   ;; source transform another chance, so it all works out OK, in a
   ;; weird roundabout way. -- WHN 2001-03-18
   (if (and (consp spec) (eq (car spec) 'quote))
-      (let ((type (specifier-type (cadr spec))))
-	(or (let ((pred (cdr (assoc type *backend-type-predicates*
+      (let ((type (careful-specifier-type (cadr spec))))
+	(or (when (not type)
+              (compiler-warn "illegal type specifier for TYPEP: ~S"
+                             (cadr spec))
+              `(%typep ,object ,spec))
+            (let ((pred (cdr (assoc type *backend-type-predicates*
 				    :test #'type=))))
 	      (when pred `(,pred ,object)))
 	    (typecase type
 	      (hairy-type
 	       (source-transform-hairy-typep object type))
+	      (negation-type
+	       (source-transform-negation-typep object type))
 	      (union-type
 	       (source-transform-union-typep object type))
 	      (intersection-type
@@ -514,35 +505,47 @@
 	    (typecase type
 	      (numeric-type
 	       (source-transform-numeric-typep object type))
-	      (sb!xc:class
+	      (classoid
 	       `(%instance-typep ,object ,spec))
 	      (array-type
 	       (source-transform-array-typep object type))
 	      (cons-type
 	       (source-transform-cons-typep object type))
+             (character-set-type
+              (source-transform-character-set-typep object type))
 	      (t nil))
 	    `(%typep ,object ,spec)))
       (values nil t)))
 
 ;;;; coercion
 
-(deftransform coerce ((x type) (* *) * :when :both)
-  (unless (constant-continuation-p type)
+(deftransform coerce ((x type) (* *) * :node node)
+  (unless (constant-lvar-p type)
     (give-up-ir1-transform))
-  (let ((tspec (specifier-type (continuation-value type))))
-    (if (csubtypep (continuation-type x) tspec)
+  (let ((tspec (ir1-transform-specifier-type (lvar-value type))))
+    (if (csubtypep (lvar-type x) tspec)
 	'x
 	;; Note: The THE here makes sure that specifiers like
 	;; (SINGLE-FLOAT 0.0 1.0) can raise a TYPE-ERROR.
-	`(the ,(continuation-value type)
+	`(the ,(lvar-value type)
 	   ,(cond
 	     ((csubtypep tspec (specifier-type 'double-float))
-	      '(%double-float x))	
+	      '(%double-float x))
 	     ;; FIXME: #!+long-float (t ,(error "LONG-FLOAT case needed"))
 	     ((csubtypep tspec (specifier-type 'float))
 	      '(%single-float x))
-	     ((csubtypep tspec (specifier-type 'simple-vector))
-	      '(coerce-to-simple-vector x))
+	     ((and (csubtypep tspec (specifier-type 'simple-vector))
+		   ;; Can we avoid checking for dimension issues like
+		   ;; (COERCE FOO '(SIMPLE-VECTOR 5)) returning a
+		   ;; vector of length 6?
+		   (or (policy node (< safety 3)) ; no need in unsafe code
+		       (and (array-type-p tspec) ; no need when no dimensions
+			    (equal (array-type-dimensions tspec) '(*)))))
+	      `(if (simple-vector-p x)
+		   x
+		   (replace (make-array (length x)) x)))
+	     ;; FIXME: other VECTOR types?
 	     (t
 	      (give-up-ir1-transform)))))))
 
+
