@@ -17,17 +17,18 @@
 #define _GC_H_
 
 extern void gc_init(void);
-
-/* Note: CMU CL had two different argument conventions for
- * collect_garbage(..), depending on whether gencgc was in use. SBCL
- * should have only one, which is automatic right now (20000814) since
- * we only support gencgc, but should also be maintained if someone
- * adds another GC, or ports one of the other CMU CL GCs like gengc. */
+extern void gc_initialize_pointers(void);
 extern void collect_garbage(unsigned last_gen);
+extern void gc_init_tables(void);
+
 
 #include "os.h"
 
 extern void set_auto_gc_trigger(os_vm_size_t usage);
 extern void clear_auto_gc_trigger(void);
 
-#endif _GC_H_
+extern int maybe_gc_pending;
+
+#include "fixnump.h"
+
+#endif /* _GC_H_ */
