@@ -218,7 +218,7 @@ static void brief_otherimm(lispobj obj)
 
     type = widetag_of(obj);
     switch (type) {
-        case BASE_CHAR_WIDETAG:
+        case CHARACTER_WIDETAG:
             c = (obj>>8)&0xff;
             switch (c) {
                 case '\0':
@@ -275,7 +275,7 @@ static void print_otherimm(lispobj obj)
 	    printf(", unknown type (0x%0x)", type);
 
     switch (widetag_of(obj)) {
-        case BASE_CHAR_WIDETAG:
+        case CHARACTER_WIDETAG:
             printf(": ");
             brief_otherimm(obj);
             break;
@@ -536,6 +536,9 @@ static void print_otherptr(lispobj obj)
 #endif
 
             case SIMPLE_BASE_STRING_WIDETAG:
+#ifdef SIMPLE_CHARACTER_STRING_WIDETAG
+	case SIMPLE_CHARACTER_STRING_WIDETAG: /* FIXME */
+#endif
                 NEWLINE_OR_RETURN;
                 cptr = (char *)(ptr+1);
                 putchar('"');
@@ -599,6 +602,9 @@ static void print_otherptr(lispobj obj)
 	    case SIMPLE_ARRAY_COMPLEX_LONG_FLOAT_WIDETAG:
 #endif
             case COMPLEX_BASE_STRING_WIDETAG:
+#ifdef COMPLEX_CHARACTER_STRING_WIDETAG
+	case COMPLEX_CHARACTER_STRING_WIDETAG:
+#endif
             case COMPLEX_VECTOR_NIL_WIDETAG:
             case COMPLEX_BIT_VECTOR_WIDETAG:
             case COMPLEX_VECTOR_WIDETAG:
@@ -642,7 +648,7 @@ static void print_otherptr(lispobj obj)
 		print_slots(weak_pointer_slots, 1, ptr);
                 break;
 
-            case BASE_CHAR_WIDETAG:
+            case CHARACTER_WIDETAG:
             case UNBOUND_MARKER_WIDETAG:
                 NEWLINE_OR_RETURN;
                 printf("pointer to an immediate?");
