@@ -39,5 +39,17 @@
     (assert (null b))
     (assert (null d))))
 
+;;; SETF of THE with VALUES.
+(let (x y)
+  (setf (the (values fixnum fixnum) (values x y))
+        (values 1 2))
+  (assert (= x 1))
+  (assert (= y 2)))
+
+;;; SETF of MACRO-FUNCTION must accept a NIL environment
+(let ((fun (constantly 'ok)))
+  (setf (macro-function 'nothing-at-all nil) fun)
+  (assert (eq fun (macro-function 'nothing-at-all nil))))
+
 ;;; success
 (quit :unix-status 104)
